/*
   MQTT sonoff switch v2.0
   AAU 04.04.2019

   Прошивка универсальная и подходит для:
   Sonoff T1/T2/T3/Touch/Basic
   Для Sonoff basic возможно подключить на GPIO14:
    датчики температуры DS18x20 (до четырех штук) или
    датчик температуры и влажности DHT22 или
    геркон или датчик движения.
    Наличие того или иного датчика определяется
   автоматически при подаче питания.
   Поддержка коротких и длинных нажатий.
   Поддержка OTA обновлений.
   web итерфейс.
   При первом включении создается точка доступа
   со страницей настроек по адресу: 192.168.4.1.
   Доступ к странице настроек в локальной сети по имени или IP

   LED:
   медленно мигает - режим AP и клиент подключен
   быстро мигает - режим STA и нет соединения с WiFi
   постоянно вкл/выкл - статус подключения к MQTT брокеру

   ==settings for sonoff T1/T2/T3/Touch==
   "Generic 8285 module"
   Upload speed: "115200"
   CPU frequency: "80MHz"
   Flash size: "1M (64K SPIFFS)"

   ==settings for sonoff basic:==
   "Generic ESP8266 module"
   Upload speed: "115200"
   CPU frequency: "80MHz"
   Flash size: "1M (64K SPIFFS)"
   Flash mode: "DOUT"
   Flash frequency: "40MHz"
   Reset Method: "ck"

   >  * VCC
        RX
        TX
        GND
        GPIO 14
*/

#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <ESP8266NetBIOS.h>
#include <PubSubClient.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
#include <Wire.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include "DHT.h"
#include <EEPROM.h>

#define VERSION "v2.0"

#define OTA_PASS ""
#define OTA_PORT 8266

//настройка порта
#define BUTTON1_PIN   0
#define BUTTON2_PIN   9
#define BUTTON3_PIN   10
#define RELAY1_PIN    12
#define RELAY2_PIN    5
#define RELAY3_PIN    4
#define INPUT1_PIN    14
#define LED_PIN       13
#define EXT_SENS_PIN  INPUT1_PIN        //куда подключены датчики DHT || DS

#define SHORT_PRESS_THRESHOLD   100     //порог антидребезга
#define LONG_PRESS_THRESHOLD    3000    //долгое нажатие

// Uncomment whatever type you're using!
//#define DHTTYPE DHT11   // DHT 11
#define DHTTYPE DHT22   // DHT 22  (AM2302), AM2321
//#define DHTTYPE DHT21   // DHT 21 (AM2301)

const String Topic1 = "/relay1";      //реле 1
const String Topic2 = "/relay2";      //реле 2
const String Topic3 = "/relay3";      //реле 3
const String Topic4 = "/longpress1";  //долгое нажатие 1
const String Topic5 = "/longpress2";  //долгое нажатие 2
const String Topic6 = "/longpress3";  //долгое нажатие 3
const String Topic7 = "/input1";      //датчик 1
const String Topic9 = "/ds18x20_1";   //ds18x20 "1"
const String Topic10 = "/ds18x20_2";  //ds18x20 "2"
const String Topic11 = "/ds18x20_3";  //ds18x20 "3"
const String Topic12 = "/ds18x20_4";  //ds18x20 "4"
const String Topic13 = "/dht_temp";   //температура с DHT
const String Topic14 = "/dht_hum";    //влажность с DHT
const String Topic15 = "/rssi";       //RSSI:
const String Topic16 = "/ip";         //IP:

volatile int inputState1 = 1;
volatile int inputState2 = 1;
volatile int inputState3 = 1;
volatile int inputState4 = 1;
volatile int inputOldState1 = 1;
volatile int inputOldState2 = 1;
volatile int inputOldState3 = 1;
volatile int inputOldState4 = 1;

volatile int relayState1 = 0;
volatile int relayState2 = 0;
volatile int relayState3 = 0;
volatile int relayOldState1 = 0;
volatile int relayOldState2 = 0;
volatile int relayOldState3 = 0;

volatile unsigned long millisSinceChange1 = 0;
volatile unsigned long millisSinceChange2 = 0;
volatile unsigned long millisSinceChange3 = 0;

volatile unsigned int brightLED = 10;
volatile unsigned long mqttPeriodSend = 60000;

bool mqttConnect = false; //MQTT connecting flag
bool wifiSTAMode;
bool printedWifiToSerial = false;

unsigned long lastMQTTCheck = -5000; //This will force an immediate check on init.
unsigned long lastMQTTSend = 0;
unsigned long lastRunConvTemp = 0;
unsigned long lastLedToggle = 0;

#define DHTPIN        EXT_SENS_PIN  // Digital pin connected to the DHT sensor
#define ONE_WIRE_BUS  EXT_SENS_PIN  // GPIO 1-wire

#define EEPROM_SIZE       256

struct eeprom_data_t {
  char nameDevice[32 + 1] = "Sonoff_Switch";
  char ssid[32 + 1] = "";
  char pass[32 + 1] = "";
  char mqttServ[64 + 1] = "";
  char mqttPort[5 + 1] = "1883";
  char mqttPeriod[3 + 1] = "60";
  char mqttLogin[32 + 1] = "";
  char mqttPass[32 + 1] = "";
  char brLed[2 + 1] = "10";
} eeprom_data;

String html_header = "<html>\
 <meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\
 <head>\
   <title>Sonoff Settings</title>\
   <style>\
     body { background-color: #cccccc; font-family: Arial, Helvetica, Sans-Serif; Color: #000088; }\
   </style>\
 </head>";

ESP8266WebServer server(80);
WiFiClient espClient;
PubSubClient client(espClient);
OneWire oneWire(ONE_WIRE_BUS);
DHT dht(DHTPIN, DHTTYPE);
DallasTemperature DSsensors(&oneWire);

uint8_t DhtDeviceFound = 0;
uint8_t DsDeviceCount = 0;

// Add 4 prepared sensors to the bus
// use the UserDataWriteBatch demo to prepare 4 different labeled sensors
struct
{
  int DSid;
  DeviceAddress DSaddr;
} T[4];
//==================================== чтение EEPROM =======================================
void readEeprom() {
  unsigned int address = 1;

  EEPROM.begin(EEPROM_SIZE);

  for (byte i = 0; i < 32; i++)eeprom_data.nameDevice[i] = char(EEPROM.read(address++));  //name len=32
  for (byte i = 0; i < 32; i++)eeprom_data.ssid[i] = char(EEPROM.read(address++));        //ssid len=32
  for (byte i = 0; i < 32; i++)eeprom_data.pass[i] = char(EEPROM.read(address++));        //pass len=32
  for (byte i = 0; i < 64; i++)eeprom_data.mqttServ[i] = char(EEPROM.read(address++));    //mqttServ len=64
  for (byte i = 0; i < 5; i++)eeprom_data.mqttPort[i] = char(EEPROM.read(address++));     //mqttPort len=5
  for (byte i = 0; i < 3; i++)eeprom_data.mqttPeriod[i] = char(EEPROM.read(address++));   //mqttPeriodSend len=3
  for (byte i = 0; i < 32; i++)eeprom_data.mqttLogin[i] = char(EEPROM.read(address++));   //mqttLogin len=32
  for (byte i = 0; i < 32; i++)eeprom_data.mqttPass[i] = char(EEPROM.read(address++));    //mqttPass len=32
  for (byte i = 0; i < 2; i++)eeprom_data.brLed[i] = char(EEPROM.read(address++));        //brLed len=2

  mqttPeriodSend = atoi(eeprom_data.mqttPeriod) * 1000;
  brightLED = atoi(eeprom_data.brLed);
}

//===========================================||=============================================
void printAddress(DeviceAddress deviceAddress)
{
  for (uint8_t i = 0; i < 8; i++)
  {
    // zero pad the address if necessary
    if (deviceAddress[i] < 16) Serial.print("0");
    Serial.print(deviceAddress[i], HEX);
  }
}

//===========================================||=============================================
void initWifi() {
  // We start by connecting to a WiFi network

  if (EEPROM.read(0) == 1) {
    // Режим STATION
    wifiSTAMode = true;
    WiFi.mode(WIFI_STA);
    wifi_station_set_hostname(eeprom_data.nameDevice);
    WiFi.begin(eeprom_data.ssid, eeprom_data.pass);
  }
  else {
    // Режим SoftAP
    wifiSTAMode = false;
    WiFi.mode(WIFI_AP);
    Serial.print("Configuring access point...");
    /* You can remove the password parameter if you want the AP to be open. */
    WiFi.softAP(eeprom_data.nameDevice);
    delay(2000);
    Serial.println("done");
    IPAddress IP = WiFi.softAPIP();
    Serial.print("AP IP address: ");
    Serial.println(IP);
  }
  server.on("/", handleRoot);
  server.on("/save", handleSave);
  server.on("/reboot", handleReboot);
  server.begin();
  Serial.println("HTTP server started");
}

//===========================================||=============================================
void checkMQTTConnection() {
  Serial.print("Checking MQTT connection: ");
  if (client.connected()) Serial.println("OK");
  else {
    if (WiFi.status() == WL_CONNECTED) {
      //Wifi connected, attempt to connect to server
      Serial.print("new connection: ");
      if (client.connect(eeprom_data.nameDevice, eeprom_data.mqttLogin, eeprom_data.mqttPass)) {
        Serial.println("connected");
        client.subscribe((String(eeprom_data.nameDevice) + Topic1).c_str());
        client.subscribe((String(eeprom_data.nameDevice) + Topic2).c_str());
        client.subscribe((String(eeprom_data.nameDevice) + Topic3).c_str());
      } else {
        Serial.print("failed, rc=");
        Serial.println(client.state());
      }
    }
    else {
      //Wifi isn't connected, so no point in trying now.
      Serial.println(" Not connected to WiFI AP, abandoned connect.");
    }
  }
  //Set the status LED to ON if we are connected to the MQTT server
  if (client.connected())
  {
    mqttConnect = 1;
    analogWrite(LED_PIN, 10 - brightLED);
  }
  else
  {
    mqttConnect = 0;
    analogWrite(LED_PIN, 10);
  }
}

//===========================================||=============================================
void MQTTcallback(char* topic, byte* payload, unsigned int length) {
  Serial.print("Message arrived [");
  Serial.print(topic);

  if (!strcmp(topic, (String(eeprom_data.nameDevice) + Topic1).c_str())) {
    if ((char)payload[0] == '1') {
      relayState1 = relayOldState1 = 1;
    }
    else if ((char)payload[0] == '0') {
      relayState1 = relayOldState1 = 0;
    }
    Serial.print(" = ");
    Serial.print((char)payload[0]);
    digitalWrite(RELAY1_PIN, relayState1);
  }

  if (!strcmp(topic, (String(eeprom_data.nameDevice) + Topic2).c_str())) {
    if ((char)payload[0] == '1') {
      relayState2 = relayOldState2 = 1;
    }
    else if ((char)payload[0] == '0') {
      relayState2 = relayOldState2 = 0;
    }
    Serial.print(" = ");
    Serial.print((char)payload[0]);
    digitalWrite(RELAY2_PIN, relayState2);
  }

  if (!strcmp(topic, (String(eeprom_data.nameDevice) + Topic3).c_str())) {
    if ((char)payload[0] == '1') {
      relayState3 = relayOldState3 = 1;
    }
    else if ((char)payload[0] == '0') {
      relayState3 = relayOldState3 = 0;
    }
    Serial.print(" = ");
    Serial.print((char)payload[0]);
    digitalWrite(RELAY3_PIN, relayState3);
  }
  Serial.println(" ]");
}

//===========================================||=============================================
void button1_ShortPress() {
  relayState1 = !relayState1; //Toggle relay state.
  digitalWrite(RELAY1_PIN, relayState1);
}
void button1_longPress() {

}

void button2_ShortPress() {
  relayState2 = !relayState2; //Toggle relay state.
  digitalWrite(RELAY2_PIN, relayState2);
}
void button2_longPress() {

}

void button3_ShortPress() {
  relayState3 = !relayState3; //Toggle relay state.
  digitalWrite(RELAY3_PIN, relayState3);
}
void button3_longPress() {

}

//===========================================||=============================================
void button1_ChangeCallback() {
  if (digitalRead(BUTTON1_PIN) == 1) {
    //Button has been released, trigger one of the two possible options.
    if (millis() - millisSinceChange1 > LONG_PRESS_THRESHOLD) {
      button1_longPress();
    }
    else if (millis() - millisSinceChange1 >= SHORT_PRESS_THRESHOLD) {
      button1_ShortPress();
    }
    else {
      //Too short to register as a press
    }
  }
  else {
    //Just been pressed - do nothing until released.
  }
  millisSinceChange1 = millis();
}

void button2_ChangeCallback() {
  if (digitalRead(BUTTON2_PIN) == 1) {
    //Button has been released, trigger one of the two possible options.
    if (millis() - millisSinceChange2 > LONG_PRESS_THRESHOLD) {
      button2_longPress();
    }
    else if (millis() - millisSinceChange2 >= SHORT_PRESS_THRESHOLD) {
      button2_ShortPress();
    }
    else {
      //Too short to register as a press
    }
  }
  else {
    //Just been pressed - do nothing until released.
  }
  millisSinceChange2 = millis();
}

void button3_ChangeCallback() {
  if (digitalRead(BUTTON3_PIN) == 1) {
    //Button has been released, trigger one of the two possible options.
    if (millis() - millisSinceChange3 > LONG_PRESS_THRESHOLD) {
      button3_longPress();
    }
    else if (millis() - millisSinceChange3 >= SHORT_PRESS_THRESHOLD) {
      button3_ShortPress();
    }
    else {
      //Too short to register as a press
    }
  }
  else {
    //Just been pressed - do nothing until released.
  }
  millisSinceChange3 = millis();
}



//=================================== опрос входов =========================================
void input_poll() {
  static int hold1 = 0, hold2 = 0, hold3 = 0, hold4 = 0;
  static int input1 = 1, input2 = 1, input3 = 1, input4 = 1;

  input1 = digitalRead(BUTTON1_PIN);
  input2 = digitalRead(BUTTON2_PIN);
  input3 = digitalRead(BUTTON3_PIN);
  if (!((DsDeviceCount || DhtDeviceFound) && (EXT_SENS_PIN == INPUT1_PIN)))input4 = digitalRead(INPUT1_PIN);


  if (inputState1 != input1) {
    if (millis() - hold1 >= LONG_PRESS_THRESHOLD) {
      inputState1 = input1;
      hold1 = millis();
    }
  }
  else {
    hold1 = millis();
  }

  if (inputState2 != input2) {
    if (millis() - hold2 >= LONG_PRESS_THRESHOLD) {
      inputState2 = input2;
      hold2 = millis();
    }
  }
  else {
    hold2 = millis();
  }

  if (inputState3 != input3) {
    if (millis() - hold3 >= LONG_PRESS_THRESHOLD) {
      inputState3 = input3;
      hold3 = millis();
    }
  }
  else {
    hold3 = millis();
  }

  if (inputState4 != input4) {
    if (millis() - hold4 >= SHORT_PRESS_THRESHOLD) {
      inputState4 = input4;
      hold4 = millis();
    }
  }
  else {
    hold4 = millis();
  }

}

//===========================================||=============================================
void setup() {
  EEPROM.begin(EEPROM_SIZE);
  if (EEPROM.read(0) == 1)readEeprom(); //если EEPROM уже записывался то считываем

  analogWriteFreq(1000);
  analogWriteRange(10);
  Serial.begin(115200);
  Serial.println();
  Serial.print("MQTT sonoff switch ");
  Serial.println(VERSION);
  Serial.print("Device name: ");
  Serial.println(eeprom_data.nameDevice);
  Serial.println("Initialising");

  pinMode(BUTTON1_PIN, INPUT_PULLUP);
  pinMode(RELAY1_PIN, OUTPUT);
  digitalWrite(RELAY1_PIN, LOW);

  pinMode(BUTTON2_PIN, INPUT_PULLUP);
  pinMode(RELAY2_PIN, OUTPUT);
  digitalWrite(RELAY2_PIN, LOW);

  pinMode(BUTTON3_PIN, INPUT_PULLUP);
  pinMode(RELAY3_PIN, OUTPUT);
  digitalWrite(RELAY3_PIN, LOW);

  pinMode(INPUT1_PIN, INPUT_PULLUP);

  pinMode(LED_PIN, OUTPUT);
  analogWrite(LED_PIN, 10);

  initWifi();

  NBNS.begin(eeprom_data.nameDevice);

  client.setServer(eeprom_data.mqttServ, atoi(eeprom_data.mqttPort));
  client.setCallback(MQTTcallback);

  ///OTA setup
  ArduinoOTA.setPort(OTA_PORT);
  ArduinoOTA.setHostname(eeprom_data.nameDevice);
  if(OTA_PASS != "")ArduinoOTA.setPassword(OTA_PASS);
  ArduinoOTA.begin();

  Serial.println("Enabling touch switch interrupt");
  attachInterrupt(digitalPinToInterrupt(BUTTON1_PIN), button1_ChangeCallback, CHANGE);
  attachInterrupt(digitalPinToInterrupt(BUTTON2_PIN), button2_ChangeCallback, CHANGE);
  attachInterrupt(digitalPinToInterrupt(BUTTON3_PIN), button3_ChangeCallback, CHANGE);

  dht.begin();

  //search DHT device
  dht.readTemperature();
  delay(2000);
  if (!isnan(dht.readTemperature())) {
    DhtDeviceFound = 1;
    Serial.print("Found DHT");
    Serial.print(DHTTYPE);
    Serial.println(" device");
  }

  DSsensors.begin();

  // count DS devices
  DsDeviceCount = DSsensors.getDeviceCount();
  if (DsDeviceCount > 4)DsDeviceCount = 4;
  if (DsDeviceCount) {
    Serial.println("Found DS18x20 devices");
    Serial.print("devices: ");
    Serial.println(DsDeviceCount);
  }

  // Read ID's per sensor
  // and put them in T array
  for (uint8_t index = 0; index < DsDeviceCount; index++)
  {
    // go through sensors
    DSsensors.getAddress(T[index].DSaddr, index);
    T[index].DSid = DSsensors.getUserData(T[index].DSaddr);
  }

  // Check all 4 sensors are set
  if (DsDeviceCount) {
    for (uint8_t index = 0; index < DsDeviceCount; index++)
    {
      Serial.print("address: ");
      printAddress(T[index].DSaddr);
      Serial.println();
    }
  }
}

//===========================================||=============================================
void loop() {
  int rssi, ds_err[4], dhtT_err, dhtH_err;
  float data;
  char ipStr[16];
  char tempDsStr[4][16], tempDhtStr[16], humDhtStr[16];

  //If we haven't printed WiFi details to Serial port yet, and WiFi now connected,
  //do so now. (just the once)
  if (!printedWifiToSerial && WiFi.status() == WL_CONNECTED) {
    Serial.println("WiFi connected");
    Serial.print("IP address: ");
    Serial.println(WiFi.localIP());
    printedWifiToSerial = true;
  }

  server.handleClient();

  if (wifiSTAMode == true) {

    // мигаем LED в режиме STA при отсутствии WiFi
    if (WiFi.status() != WL_CONNECTED) {
      if (millis() - lastLedToggle >= 100) {
        lastLedToggle = millis();
        if (digitalRead(LED_PIN))digitalWrite(LED_PIN, LOW);
        else digitalWrite(LED_PIN, HIGH);
      }
    }

    if (millis() - lastMQTTCheck >= 5000) {
      checkMQTTConnection();
      lastMQTTCheck = millis();
    }

    //Handle any pending MQTT messages
    client.loop();

    //Handle any pending OTA SW updates
    ArduinoOTA.handle();

    //== Read inputs ==
    input_poll();

    if (mqttConnect) {
      //=========== Relay1 state is updated via the interrupt. ===============
      if (relayState1 != relayOldState1) {
        Serial.print("Changing state relay1 to ");
        Serial.println(relayState1);
        relayOldState1 = relayState1;
        Serial.print("Sending MQTT status update ");
        Serial.print(relayState1);
        Serial.print(" to ");
        Serial.println(String(eeprom_data.nameDevice) + Topic1);
        client.publish((String(eeprom_data.nameDevice) + Topic1).c_str(), relayState1 == 0 ? "0" : "1");
      }
      //=================== Long press button1 updated ========================
      if (inputState1 != inputOldState1) {
        inputOldState1 = inputState1;
        Serial.print("Sending MQTT status update ");
        Serial.print(!inputState1);
        Serial.print(" to ");
        Serial.println(String(eeprom_data.nameDevice) + Topic4);
        client.publish((String(eeprom_data.nameDevice) + Topic4).c_str(), inputState1 == 0 ? "1" : "0");
      }

      ///=========== Relay2 state is updated via the interrupt. ===============
      if (relayState2 != relayOldState2) {
        Serial.print("Changing state relay2 to ");
        Serial.println(relayState2);
        relayOldState2 = relayState2;
        Serial.print("Sending MQTT status update ");
        Serial.print(relayState2);
        Serial.print(" to ");
        Serial.println(String(eeprom_data.nameDevice) + Topic2);
        client.publish((String(eeprom_data.nameDevice) + Topic2).c_str(), relayState2 == 0 ? "0" : "1");
      }
      //=================== Long press button2 updated ========================
      if (inputState2 != inputOldState2) {
        inputOldState2 = inputState2;
        Serial.print("Sending MQTT status update ");
        Serial.print(inputState2);
        Serial.print(" to ");
        Serial.println(String(eeprom_data.nameDevice) + Topic5);
        client.publish((String(eeprom_data.nameDevice) + Topic5).c_str(), inputState2 == 0 ? "1" : "0");
      }

      //=========== Relay3 state is updated via the interrupt. ===============
      if (relayState3 != relayOldState3) {
        Serial.print("Changing state relay3 to ");
        Serial.println(relayState3);
        relayOldState3 = relayState3;
        Serial.print("Sending MQTT status update ");
        Serial.print(relayState3);
        Serial.print(" to ");
        Serial.println(String(eeprom_data.nameDevice) + Topic3);
        client.publish((String(eeprom_data.nameDevice) + Topic3).c_str(), relayState3 == 0 ? "0" : "1");
      }
      //=================== Long press button3 updated ========================
      if (inputState3 != inputOldState3) {
        inputOldState3 = inputState3;
        Serial.print("Sending MQTT status update ");
        Serial.print(inputState3);
        Serial.print(" to ");
        Serial.println(String(eeprom_data.nameDevice) + Topic6);
        client.publish((String(eeprom_data.nameDevice) + Topic6).c_str(), inputState3 == 0 ? "1" : "0");
      }

      //===============отправляем состояние входа1 по MQTT====================
      if (inputState4 != inputOldState4) {
        inputOldState4 = inputState4;
        Serial.print("Sending MQTT status update ");
        Serial.print(inputState4);
        Serial.print(" to ");
        Serial.println(String(eeprom_data.nameDevice) + Topic7);
        client.publish((String(eeprom_data.nameDevice) + Topic7).c_str(), inputState4 == 0 ? "0" : "1");
      }

      //========== запуск преобразования температуры DS или DHT ===========
      if (millis() - lastRunConvTemp >= mqttPeriodSend - 2000) {
        lastRunConvTemp = millis();
        if (DsDeviceCount) DSsensors.requestTemperatures();
        if (DhtDeviceFound) dht.readTemperature();
      }
      
      //===================== Periodic status updates =======================
      if (millis() - lastMQTTSend >= mqttPeriodSend) {
        lastMQTTSend = millis();

        if (DhtDeviceFound) {
          data = dht.readTemperature();
          if (isnan(data)) {
            dhtT_err = 1;    //флаг ошибки dht
          }
          else {
            dhtT_err = 0; //флаг ошибки dht
            dtostrf(data, 1, 1, tempDhtStr);
          }
          data = dht.readHumidity();
          if (isnan(data)) {
            dhtH_err = 1;
          }
          else {
            dhtH_err = 0;
            dtostrf(data, 1, 0, humDhtStr);
          }
        }

        for (int i = 0; i < DsDeviceCount; i++)
        {
          data = DSsensors.getTempC(T[i].DSaddr);
          if (data == -127) {
            ds_err[i] = 1;    //флаг ошибки ds
          }
          else {
            ds_err[i] = 0;
            dtostrf(data, 1, 1, tempDsStr[i]);
          }
        }
        rssi = WiFi.RSSI();
        IPAddress ip = WiFi.localIP();
        sprintf (ipStr, "%d.%d.%d.%d", ip[0], ip[1], ip[2], ip[3]);
        Serial.println("Periodic data sending:");
        Serial.print(relayState1);
        Serial.print(" to ");
        Serial.println(String(eeprom_data.nameDevice) + Topic1);
        Serial.print(relayState2);
        Serial.print(" to ");
        Serial.println(String(eeprom_data.nameDevice) + Topic2);
        Serial.print(relayState3);
        Serial.print(" to ");
        Serial.println(String(eeprom_data.nameDevice) + Topic3);
        if (DhtDeviceFound) {
          if (dhtT_err == 0) {
            Serial.print(tempDhtStr);
            Serial.print(" to ");
            Serial.println(String(eeprom_data.nameDevice) + Topic13);
          }
          if (dhtT_err == 0) {
            Serial.print(humDhtStr);
            Serial.print(" to ");
            Serial.println(String(eeprom_data.nameDevice) + Topic14);
          }
        }
        if (DsDeviceCount) {
          for (int i = 0; i < DsDeviceCount; i++)
          {
            if (ds_err[i] == 0) {
              Serial.print(tempDsStr[i]);
              Serial.print(" to ");
              if (i == 0)Serial.println(String(eeprom_data.nameDevice) + Topic9);
              if (i == 1)Serial.println(String(eeprom_data.nameDevice) + Topic10);
              if (i == 2)Serial.println(String(eeprom_data.nameDevice) + Topic11);
              if (i == 3)Serial.println(String(eeprom_data.nameDevice) + Topic12);
            }
          }
        }
        lastRunConvTemp = lastMQTTSend;//синхронизируем запуск преобразования DS и DHT
        Serial.print(rssi);
        Serial.print(" to ");
        Serial.println(String(eeprom_data.nameDevice) + Topic15);
        Serial.print(ip);
        Serial.print(" to ");
        Serial.println(String(eeprom_data.nameDevice) + Topic16);
        client.publish((String(eeprom_data.nameDevice) + Topic1).c_str(), relayState1 == 0 ? "0" : "1");
        client.publish((String(eeprom_data.nameDevice) + Topic2).c_str(), relayState2 == 0 ? "0" : "1");
        client.publish((String(eeprom_data.nameDevice) + Topic3).c_str(), relayState3 == 0 ? "0" : "1");
        if (DsDeviceCount > 0 && ds_err[0] == 0)client.publish((String(eeprom_data.nameDevice) + Topic9).c_str(), tempDsStr[0]);
        if (DsDeviceCount > 1 && ds_err[1] == 0)client.publish((String(eeprom_data.nameDevice) + Topic10).c_str(), tempDsStr[1]);
        if (DsDeviceCount > 2 && ds_err[2] == 0)client.publish((String(eeprom_data.nameDevice) + Topic11).c_str(), tempDsStr[2]);
        if (DsDeviceCount > 3 && ds_err[3] == 0)client.publish((String(eeprom_data.nameDevice) + Topic12).c_str(), tempDsStr[3]);
        if (DhtDeviceFound && dhtT_err == 0)client.publish((String(eeprom_data.nameDevice) + Topic13).c_str(), tempDhtStr);
        if (DhtDeviceFound && dhtH_err == 0)client.publish((String(eeprom_data.nameDevice) + Topic14).c_str(), humDhtStr);
        client.publish((String(eeprom_data.nameDevice) + Topic15).c_str(), String(rssi).c_str());
        client.publish((String(eeprom_data.nameDevice) + Topic16).c_str(), ipStr);
      }
      //======================================================================
    }
  }
  else {// мигаем LED в режиме AP при подключении клиента
    if (millis() - lastLedToggle >= 1000) {
      lastLedToggle = millis();
      if (digitalRead(LED_PIN) && wifi_softap_get_station_num())digitalWrite(LED_PIN, LOW);
      else digitalWrite(LED_PIN, HIGH);
    }
  }
  delay(10);
}

//===========================================||=============================================
void handleRoot() {
  String str = "";
  str += html_header;
  str += "<body>\
  <form method=\"POST\" action=\"save\">\
   <h1 style=\"text-align: center;\"><font size=\"6\"><code>Sonoff switch settings</code></font></h1>\
    <p align=\"center\">-= WI-FI settings =-&nbsp;</p>\
     <p align=\"center\">Name:&nbsp; &nbsp;<input maxlength=\"32\" name=\"name\" size=\"20\" type=\"text\" value=\"" + String(eeprom_data.nameDevice) + "\"></p>\
      <p align=\"center\">SSID:&nbsp; &nbsp; &nbsp;<input maxlength=\"32\" name=\"ssid\" size=\"20\" type=\"text\" value=\"" + String(eeprom_data.ssid) + "\"></p>\
       <p align=\"center\">Pass:&nbsp; &nbsp; &nbsp;<input maxlength=\"32\" name=\"pass\" size=\"20\" type=\"password\" value=\"" + String(eeprom_data.pass) + "\"></p>\
        <p align=\"center\">-= MQTT settings =-&nbsp;</p>\
         <p align=\"center\">Server:&nbsp; &nbsp;<input maxlength=\"64\" name=\"mqttserv\" type=\"text\" value=\"" + String(eeprom_data.mqttServ) + "\"></p>\
          <p align=\"center\">Port:&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;<input maxlength=\"5\" name=\"mqttport\" size=\"5\" type=\"text\" value=\"" + String(eeprom_data.mqttPort) + "\"></p>\
           <p align=\"center\">Period send (4-999s):&nbsp; &nbsp;<input maxlength=\"3\" name=\"period\" size=\"5\" type=\"text\" value=\"" + String(eeprom_data.mqttPeriod) + "\"></p>\
            <p align=\"center\">Login:&nbsp; &nbsp; &nbsp;<input maxlength=\"32\" name=\"mqttlogin\" size=\"20\" type=\"text\" value=\"" + String(eeprom_data.mqttLogin) + "\"></p>\
             <p align=\"center\">Pass:&nbsp; &nbsp; &nbsp;<input maxlength=\"32\" name=\"mqttpass\" size=\"20\" type=\"password\" value=\"" + String(eeprom_data.mqttPass) + "\"></p>\
              <p align=\"center\">Bright LED (1-10):&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;<input maxlength=\"2\" name=\"brLed\" size=\"2\" type=\"text\" value=\"" + String(eeprom_data.brLed) + "\"></p>\
               <p align=\"center\"><a href=\"save\"><input type=\"submit\" value=\"Save\"></a>&nbsp; &nbsp;<a href=\"reboot\"><input name=\"reboot\" type=\"button\" value=\"Reboot\"></a></p>\
                <p align=\"center\"><font face=\"monospace\" size=\"3\">uptime: ";
  str += humanTimeMillis(millis());
  str += "&nbsp;</font></p>\
               </form>\
         </body>\
    </html>";
  server.send ( 200, "text/html", str );
}

//===========================================||=============================================
void handleSave() {
  unsigned int address = 0;
  String str;
  unsigned char* buf = new unsigned char[65];

  EEPROM.begin(EEPROM_SIZE);

  EEPROM.write(address++, 1); //установим флаг, что модуль сконфигурирован

  str = server.arg("name");
  str.getBytes(buf, 32 + 1);
  for (byte i = 0; i < 32; i++)EEPROM.write(address++, buf[i]); //name 32

  str = server.arg("ssid");
  str.getBytes(buf, 32 + 1);
  for (byte i = 0; i < 32; i++)EEPROM.write(address++, buf[i]); //ssid 32

  str = server.arg("pass");
  str.getBytes(buf, 32 + 1);
  for (byte i = 0; i < 32; i++)EEPROM.write(address++, buf[i]); //pass 32

  str = server.arg("mqttserv");
  str.getBytes(buf, 64 + 1);
  for (byte i = 0; i < 64; i++)EEPROM.write(address++, buf[i]); //mqttServ 64

  str = server.arg("mqttport");
  str.getBytes(buf, 5 + 1);
  for (byte i = 0; i < 5; i++)EEPROM.write(address++, buf[i]); //mqttPort 5

  str = server.arg("period");
  if (atoi(str.c_str()) < 4 || atoi(str.c_str()) > 999)str = "60";
  str.getBytes(buf, 3 + 1);
  for (byte i = 0; i < 3; i++)EEPROM.write(address++, buf[i]); //mqttPeriodSend 5

  str = server.arg("mqttlogin");
  str.getBytes(buf, 32 + 1);
  for (byte i = 0; i < 32; i++)EEPROM.write(address++, buf[i]); //mqttLogin 32

  str = server.arg("mqttpass");
  str.getBytes(buf, 32 + 1);
  for (byte i = 0; i < 32; i++)EEPROM.write(address++, buf[i]); //mqttPass 32

  str = server.arg("brLed");
  if (atoi(str.c_str()) < 1 || atoi(str.c_str()) > 10)str = "10";
  str.getBytes(buf, 2 + 1);
  for (byte i = 0; i < 2; i++)EEPROM.write(address++, buf[i]); //brLed 2

  Serial.println("Settings saved to EEPROM");

  EEPROM.end();

  readEeprom();

  if (server.arg("ssid") != "") {
    str = "";
    str += html_header;
    str += "<body>";
    str += "<p align=\"center\">Configuration saved in FLASH</br>\
   Changes applied after reboot</br></br>\
   <a href=\"/\">Return</a> to settings page</br></p>";
  }
  else {
    EEPROM.begin(EEPROM_SIZE);
    EEPROM.write(0, 0);
    EEPROM.end();
    str = "";
    str += html_header;
    str += "<body>";
    str += "<p align=\"center\">Enter SSID Your WI-FI net</br>\
   <a href=\"/\">Return</a> to settings page</br></p>";
  }
  str += "</body></html>";
  server.send ( 200, "text/html", str );
}

//===========================================||=============================================
void handleReboot() {
  String str;

  str = "";
  str += html_header;
  str += "<body>";
  str += "<p align=\"center\">Rebooting...</br>\
   <a href=\"/\">Return</a> to settings page</br></p>";
  str += "</body></html>";
  server.send ( 200, "text/html", str );
  delay(1000);
  ESP.reset();
}

//===================================humanTimeMillis========================================
String humanTimeMillis(unsigned long milli) {

  String s;

  unsigned long secs = milli / 1000, mins = secs / 60;
  unsigned int hours = mins / 60, days = hours / 24;
  milli -= secs * 1000;
  secs -= mins * 60;
  mins -= hours * 60;
  hours -= days * 24;
  s += days != 0 ?  (String)days : "";
  s += days != 0 ?  "d " : "";
  s += hours != 0 ?  (String)hours : "";
  s += hours != 0 ?  ":" : "";
  s += mins > 9 ?  "" : "0";
  s += mins;
  s += ":";
  s += secs > 9 ?  "" : "0";
  s += secs;

  return s;
}

//===========================================||=============================================
